package Requester::ping;

use Requester::SlowRequester;
use FileHandle;
use Socket;
use strict;
use CWorker;
use MicroTime;

our( @ISA );
@ISA = qw(Requester::SlowRequester);

my $requester = new Requester::ping();
my $icmp_proto = (getprotobyname('icmp'))[2];
unless( defined $icmp_proto ) {
    BigSister::common::log( "err", "icmp protocol is unknown" );
}
my $sequence = 1;
my $pid = $$ & 0xffff;
my $icmp_struct = "C2 S3 A";  # Structure of a minimal ICMP packet
my $ICMP_ECHOREPLY = 0; # ICMP packet types
my $ICMP_ECHO = 8;

sub new {
    my ($this, $args ) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $self->register( "ping" );
    return $self;
}


sub startup {
    my( $self, $gets, $indexes ) = @_;

    my $fh = new FileHandle();
    unless( socket( $fh, PF_INET(), SOCK_RAW(), $icmp_proto ) ) {
    	return undef;
    }
    return $fh;
}


sub send_ping {
    my( $self, $srq ) = @_;

    my $subcode = 0;        # No ICMP subcode for ECHO and ECHOREPLY

    $sequence = ($sequence + 1) % 0xffff;
    print STDERR "sending ping request $sequence to ".inet_ntoa( $srq->{"ping.ip"} )."\n" if( $BigSister::common::dl > 3 );
    $srq->{"ping.sendcount"}++;
    my $env = $srq->{"ping.env"};
    $srq->{"ping.sequence"} = $sequence;
    my $checksum = 0;
    my $data_size = 20;
    my $data = "\0" x $data_size;
    my $msg = pack($icmp_struct . $data_size, $ICMP_ECHO, $subcode,
    		   $checksum, $pid, $sequence, $data );
    $checksum = checksum( $msg );
    $msg = pack($icmp_struct . $data_size, $ICMP_ECHO, $subcode,
    	   	$checksum, $pid, $sequence, $data );
    my $saddr = sockaddr_in(0,$srq->{"ping.ip"});
    send( $srq->{"SRQ.iohandle"}, $msg, 0, $saddr );
    $srq->{"ping.sendtime"} = MicroTime::gettimeofday();
    $srq->{"timeout"} = MicroTime::gettimeofday() + 5;
}


sub do_raw {
    my( $self, $srq ) = @_;
    my $fh = $srq->{"SRQ.iohandle"};
    my $rmask = undef;
    vec( $rmask, fileno($fh), 1 ) = 1;
    return unless select( $rmask, undef, undef, 0 );
    my $recv_msg;
    my( $from_saddr, $from_port, $from_ip );
    eval {
	$from_saddr = recv($fh, $recv_msg, 1500, 0);
	return unless( $from_saddr );
	($from_port, $from_ip) = sockaddr_in($from_saddr);
    };
    return if( $@ );

    my $len_msg = 28;
    my ($from_type, $from_subcode, $from_chk,
	$from_pid, $from_seq, $from_msg) =
	 unpack($icmp_struct . 20, 
	   substr($recv_msg, length($recv_msg) - $len_msg, $len_msg));

    if (($from_type == $ICMP_ECHOREPLY) &&
        ($from_pid == $pid) ) {
	print STDERR "received ping answer from ".inet_ntoa( $from_ip )."\n" 
	  if( $BigSister::common::dl > 5 );
	$self->submit_gotping( $srq, $from_ip, $from_seq );
    }
}


sub submit_gotping {
    my( $self, $srq, $ip, $seq ) = @_;

    $self->gotping( $srq, $ip, $seq );
#    foreach my $worker (@{CWorker::active_workers()}) {
#        if( UNIVERSAL::isa( $worker, "SlowReqWorker" )
#	    && UNIVERSAL::isa( $worker->{"SRQ.requester"}, "Requester::ping" )
#	) {
#	    $worker->{"SRQ.requester"}->gotping( $worker, $ip, $seq );
#	}
#    }
}


sub gotping {
    my( $self, $srq, $ip, $seq ) = @_;

    return unless( ($srq->{"ping.ip"} eq $ip) && ($srq->{"ping.sequence"} == $seq) );
    print STDERR "seen matching ping answer from ".inet_ntoa( $ip )."\n"
       if( $BigSister::common::dl > 3 );
    # avoid counting the same packet twice (shouldn't happen, anyway)
    $srq->{"ping.sequence"} = -1;
    if( $srq->{"ping.sendcount"} > 0 ) {
	$srq->{"ping.recvcount"}++;
	my $time = MicroTime::difftime( $srq->{"ping.sendtime"} );
	$srq->{"ping.totaltime"} += $time;
	$srq->{"ping.mintime"} = $time
	    if( (! defined $srq->{"ping.mintime"}) || ($srq->{"ping.mintime"} > $time) );
	$srq->{"ping.maxtime"} = $time
	    if( (! defined $srq->{"ping.maxtime"}) || ($srq->{"ping.maxtime"} < $time) );
    }
    $self->schedule_next( $srq );
}


sub timed_out {
    my( $self, $srq ) = @_;

    if( $srq->{"filenumber"} ) {
        # lost a packet
	$self->schedule_next( $srq );
	return;
    }
    else {
        # delay to next ping is over
	$srq->{"filenumber"} = $srq->{"SRQ.iohandle"}->fileno();
	$self->send_ping( $srq );
    }
}



sub schedule_next {
    my( $self, $srq ) = @_;

    print STDERR "ping schedule_next()\n"
       if( $BigSister::common::dl > 5 );
    if( $srq->{"ping.sendcount"} < $srq->{"ping.wantcount"} ) {
        $srq->{"timeout"} = MicroTime::gettimeofday() + 0.5;
	$srq->{"filenumber"} = undef;
	return;
    }
    my $info = $srq->{"Test.info"};
    if( $srq->{"ping.recvcount"} ) {
        $srq->{"ping.totaltime"} /= $srq->{"ping.recvcount"};
    }
    $info->{"0"} = {
        "sendcount" => $srq->{"ping.sendcount"},
	"recvcount" => $srq->{"ping.recvcount"},
	"maxroundtrip" => $srq->{"ping.maxtime"},
	"minroundtrip" => $srq->{"ping.mintime"},
	"avgroundtrip" => $srq->{"ping.totaltime"},
    };
    $srq->status( 3, "finished OK" );
}



sub checksum
{
    my ( $msg ) = @_;

    my $len_msg = length($msg);
    my $num_short = $len_msg / 2;
    my $chk = 0;
    foreach my $short (unpack("S$num_short", $msg))
    {
        $chk += $short;
    }                                           # Add the odd byte in
    $chk += unpack("C", substr($msg, $len_msg - 1, 1)) if $len_msg % 2;
    $chk = ($chk >> 16) + ($chk & 0xffff);      # Fold high into low
    return(~(($chk >> 16) + $chk) & 0xffff);    # Again and complement
}




sub setup {
    my( $self, $srq, @other_args ) = @_;

    $self->SUPER::setup( $self, $srq, @other_args );
    $srq->{"ping.env"} = $self->{"env"};
    my $host = $self->{"env"}->{"nodeinstance"}->{"host"};
    my $ip = inet_aton($host);
    $srq->{"ping.ip"} = $ip;
    unless( $ip ) {
        $self->status( 0, "HOST UNKNOWN" );
	return;
    }
    $srq->{"ping.sendcount"} = $srq->{"ping.recvcount"} = 0;
    $srq->{"ping.sendcount"} = -1;
    $srq->{"ping.totaltime"} = 0;
    $srq->{"ping.maxtime"} = $srq->{"ping.mintime"} = undef;
    $srq->{"ping.wantcount"} = $self->{"env"}->expand_value( "count" ) || 4;
    $self->send_ping( $srq );
    $srq->consumes(0);
    $srq->criticalness(3);
}



sub available {
    # well, let's see if we'll find something better ...
    return(0) unless( defined $icmp_proto );
    my $fh = new FileHandle();
    return(0) unless( socket( $fh, PF_INET(), SOCK_RAW(), $icmp_proto ) );
    close $fh;
    return(1);
}


sub do {
}

1;

