#!/usr/bin/perl


package Requester::win32perf;

use Requester::Requester;

our( @ISA );
@ISA = qw(Exporter Requester::Requester);

use Win32::PerfLib;
use strict;

my $requester = new Requester::win32perf;
my $PERF_AVERAGE_TIMER = 0x30020400;


my %countertype_fields = (
    "size" => 0x300,
    "type" => 0xC00,
    "numberdisplay" => 0x70000,
    "countertype" => 0x70000,
    "texttype" => 0x70000,
    "timer_subtype" => 0x300000,
    "calculation" => 0x3C00000,
    "display" => 0x70000000
);


my %countertypes = (
    "size" => {
	0 => "size_dword",
	0x100 => "size_large",
	0x200 => "size_zero",
	0x300 => "size_variable"
    },

    "type" => {
	0=> "type_number" => 0,
	0x400 => "type_counter",
	0x800 => "type_text",
	0xc00 => "type_zero"
    },

	"numberdisplay" => {
	    0 => "number_hex",
	    0x10000 => "number_decimal",
	    0x20000 => "number_dec_1000"
	},

	"countertype" => {
	    0 => "counter_value",
	    0x10000 => "counter_rate",
	    0x20000 => "counter_fraction",
	    0x30000 => "counter_base",
	    0x40000 => "counter_elapsed",
	    0x50000 => "counter_queuelen",
	    0x60000 => "counter_histogram",
	    0x70000 => "counter_precision",
	},

	"texttype" => {
	    0 => "text_unicode",
	    0x10000 => "text_ascii"
	},

    "timer_subtype" => {
	0 => "timer_tick",
	0x100000 => "timer_100ns",
	0x200000 => "timer_object"
    },

    "calculation" => {
	0x400000 => "delta_counter",
	0x800000 => "delta_base",
	0x1000000 => "inverse_counter",
	0x2000000 => "multi_counter"
    },

    "display" => {
	0 => "display_no_suffix",
	0x10000000 => "display_per_sec",
	0x20000000 => "display_percent",
	0x30000000 => "display_seconds",
	0x40000000 => "display_noshow"
    }
);


my $i;
foreach $i (keys %countertypes) {
    my $j;
    foreach $j (keys %{$countertypes{$i}}) {
	$countertypes{$i}->{$countertypes{$i}->{$j}} = $j;
    }
}


if( $0 =~ /win32perf.pm/ ) {
    test( @ARGV );
}


sub test {
    my( @objs ) = @_;
    my %counternames;
    my %helptexts;
    Win32::PerfLib::GetCounterNames(undef,\%counternames);
    Win32::PerfLib::GetCounterHelp(undef,\%helptexts); 
    my %r_counternames = map { $counternames{$_} => $_ } keys %counternames;

    my $i;
    foreach $i (keys %countertypes) {
	my $j;
	foreach $j (keys %{$countertypes{$i}}) {
	    $countertypes{$i}->{$countertypes{$i}->{$j}} = $j;
	}
    }

    #my $key;
    #foreach $key (sort { $a <=> $b } keys %counternames) {
    #    print "Counter $key: ".($counternames{$key})."\n";
    #}

    unless( @objs ) {
	my @objs = ( "Processor",
		    "Redirector",
		    "Server",
		    "Server Work Queues",
		    "System",
		    "TCP",
		    "Telephony",
		    "Thread",
		    "UDP",
		    "WINS Server",
		    "Process",
		    "PhysicalDisk",
		    "Paging File",
		    "Objects",
		    "Network Segment",
		    "Network Interface",
		    "NBT Connection",
		    "Memory",
		    "LogicalDisk",
		    "IP",
		    "ICMP",
		    "Cache",
		    "Browser" );

	@objs = ("LogicalDisk");
    }
    my $perflib = new Win32::PerfLib(undef);

    my $i;
    my $proc_ref = {};
    my $lib = Requester::win32perf->new();
    my $handle = $lib->_handle( 1, "" );
    my $objhash = $handle->{"objects"} = {};
    foreach my $obj (@objs) { $objhash->{$obj} = 1; }

    for( $i=0; $i<5; $i++ ) {
	print "============================== $i =====================\n";
        my $info = $lib->getinfo();
	foreach my $instance (keys %$info) {
	    my $vars = $info->{$instance};
	    foreach my $var ( sort keys %$vars ) {
	        print "$var.$instance = ".($vars->{$var})."\n";
	    }
	}
	sleep 1;
	print "\n";
	next;

	my $previous = $proc_ref;
	$proc_ref = {};
	my $obj;
	my $ref;

	$ref = $proc_ref;
	foreach $obj ( @objs ) {
	    my $id = $r_counternames{$obj};
	    $ref->{$obj} = {};
	    $perflib->GetObjectList($id, $ref->{$obj});
	}
	foreach $obj (keys %$proc_ref) {
	    my $inst;
	    my $obid;
	    if( $previous->{$obj} ) {
		my $timebase = $proc_ref->{$obj}->{"PerfTime100nSec"};
		my $timebase0 = $previous->{$obj}->{"PerfTime100nSec"};
		print "time base in 100nsec: ".($timebase-$timebase0)."\n";
		my $timebase = $proc_ref->{$obj}->{"PerfTime"};
		my $timebase0 = $previous->{$obj}->{"PerfTime"};
		print "time base in sec: ".($timebase-$timebase0)."\n";
	    }
	    foreach $obid (keys %{$proc_ref->{$obj}->{"Objects"}}) {
	        print "Computing $obid\n";
		foreach $inst (keys %{$proc_ref->{$obj}->{"Objects"}->{$obid}->{"Instances"}}) {
		    print "  Instance $inst\n";
		    my %var;
		    $lib->compute( $handle, $proc_ref->{$obj}, $previous->{$obj}, $obid, $inst, $obj, \%var );
		    while( my($key,$val) = each %var ) {
		        print "perflib.$key.$inst = $val\n";
		    }
		}
	    }
	    print "\n\n\n\nObject: $obj\n\n";
	    $lib->print_hash( $proc_ref->{$obj} );
	}
	sleep( 1 );
    }

    $perflib->Close();
}



sub new {
    my ($this, $args ) = @_;
    my  $class = ref($this) || $this;
    my  $self = {};

    bless($self, $class);
    $self->{"PerfLib::handles"} = {};
    $self->register( "win32perf" );
    print "Win32perf starting\n";
    return $self;
}



sub _handle {
    my( $self, $frequency, $host ) = @_;

    my $handle = $self->{"PerfLib::handles"}->{"$host.$frequency"} || {};
    $self->{"PerfLib::handles"}->{"$host.$frequency"} = $handle;
    $self->{"PerfLib::currenthandle"} = $handle;

    unless( defined $handle->{"counternames"} ) {
	my %counternames;
	my %helptexts;
	Win32::PerfLib::GetCounterNames($host,\%counternames);
	Win32::PerfLib::GetCounterHelp($host,\%helptexts); 
	my %r_counternames = map { $counternames{$_} => $_ } keys %counternames;

	$handle->{"counternames"} = \%counternames;
	$handle->{"helptexts"} = \%helptexts;
	$handle->{"r_counternames"} = \%r_counternames;
	$handle->{"objects"} = {};
	$handle->{"perflib"} = new Win32::PerfLib($host);
	$handle->{"previous"} = {};
	$handle->{"host"} = $host;
    }
     
    unless( $handle->{"perflib"} && %{$handle->{"counternames"}} ) {
	BigSister::common::log( "err", "cannot open perflib to host $host" );
    }
    return $handle;
}


sub request {
    my( $self, $env, $domain, $variables ) = @_;

    my $host = $env->{"nodeinstance"}->{"host"};
    my $frequency = $env->{"sched_frequency"};
    my $handle = $self->_handle( $frequency, $host );

    my %objects = ();
    foreach my $get (@$variables) {
        my $object = $get;
	$object =~ s/^\@//;
	$object =~ s/^(.*?)\..*/$1/;
	$objects{$object} = 1;
    }
    $handle->{"objects"} = \%objects;
    $self->SUPER::request( $env, $domain, $variables );
}


sub getinfo {
    my( $self ) = @_;

    my $handle = $self->{"PerfLib::currenthandle"};
    my $info = {};
    foreach my $object (keys %{$handle->{"objects"}}) {
	my %vars;
	my $id = $handle->{"r_counternames"}->{$object};
	my $ref = {};
	print STDERR "Requester::PerfLib: getting object $object($id)\n" if( $BigSister::common::dl>4 );
	$handle->{"perflib"}->GetObjectList($id, $ref);
	my $instance;
	$vars{"perflib.$object.Instances"} = [ (keys %{$ref->{"Objects"}->{$id}->{"Instances"}}) ];
	unless( @{$vars{"perflib.$object.Instances"}} ) {
	    $vars{"perflib.$object.Instances"} = [0];
	    $ref->{"Objects"}->{$id}->{"Instances"} = {
		0 => { 
		    "Counters" => $ref->{"Objects"}->{$id}->{"Counters"}
		}
	    };
	}
	foreach $instance (@{$vars{"perflib.$object.Instances"}}) {
	    unless( defined $info->{$instance} ) {
		$info->{$instance} = {};
	    }
	    print "  computing instance $instance\n" if( $BigSister::common::dl>5 );
	    my %vars;
	    $self->compute( $handle, $ref, $handle->{"previous"}->{$object},
		$id, $instance, "$object", \%vars );
	    my( $key, $val );
	    while( ($key,$val) = each %vars ) {
		$info->{$instance}->{$key} = $val;
		print STDERR "$key.$instance = $val\n" if( $BigSister::common::dl > 6 );
	    }
	}
#	$self->print_hash( $ref );
	$handle->{"previous"}->{$object} = $ref;
    }
    return($info);
}






sub print_hash {
    my( $self, $ref, $indent ) = @_;
    my( $key, $val );
    my $counternames = $self->{"counternames"};
    my $helptexts = $self->{"helptexts"};

    ($indent = 0) unless( $indent );
    foreach $key (sort keys %$ref) {
	$val = $ref->{$key};
	print " " x $indent;
	print $key."\t";
	if( ref($val) eq "HASH" ) {
	    print "\n";
	    $self->print_hash( $val, $indent+2 );
	    print "\n";
	}
	else {
	    (($key eq "CounterNameTitleIndex") && (defined $counternames->{$val})) && ($val=$counternames->{$val});
	    (($key eq "CounterHelpTitleIndex") && (defined $helptexts->{$val})) && ($val=$helptexts->{$val});
	    if( ($key eq "CounterType") ) {
		my $portion;
		my $type = $val;

		$val = "";
		foreach $portion (keys %countertype_fields) {
		    next if( grep( $portion eq $_, "texttype", "display", "numberdisplay", "size" ) );
		    my $type = $type & $countertype_fields{$portion};
		    my $flag;

		    if( $flag = ($countertypes{$portion}->{$type}) ) {
			$val .= " $flag";
		    }
		    else {
			my $lis;

			foreach $lis (keys %{$countertypes{$portion}}) {
			    if( $type & $lis == $lis ) {
				$val .= " ".$countertypes{$portion}->{$lis};
			    }
			}
		    }
		}
	    }
	    print $val;
	}
	print "\n";
    }
}



sub compute {
    my( $self, $handle, $current, $previous, $object, $instance, $prefix, $variables ) = @_;
    my $counternames = $handle->{"counternames"};
    my $timediff = 0;
    my $timediff100 = 0;
    my( $countertype, $prevcounters, $id );
    my $counters = $current->{"Objects"}->{$object}->{"Instances"}->{$instance}->{"Counters"};
    my $perffreq = $current->{"PerfFreq"};

#    $prefix .= ".$instance";
    $variables->{$prefix.".InstanceName"} = $current->{"Objects"}->{$object}->{"Instances"}->{$instance}->{"Name"};
    if( $previous ) {
	my $timediff = $current->{"PerfTime"} - $previous->{"PerfTime"};
	my $timediff100 = $current->{"PerfTime100nSec"} - $previous->{"PerfTime100nSec"};
	$prevcounters = $previous->{"Objects"}->{$object}->{"Instances"}->{$instance}->{"Counters"};
    }
    foreach $id (keys %$counters) {
	my $counter = $counters->{$id};
	my $name = $counternames->{$counter->{"CounterNameTitleIndex"}};
	$name =~ tr/\. /__/;
	my $varname = $prefix.".".$name;
	print "var $varname\n" if( $BigSister::common::dl > 6 );
#	$BigSister::common::dl = 10 if( $name eq "% Disk Time" );
	my $typecode = $counter->{"CounterType"};
	my $type = $countertypes{"countertype"}->{$typecode & $countertype_fields{"countertype"}};
	my $valtype = $countertypes{"type"}->{$typecode & $countertype_fields{"type"}};
	my $time = $countertypes{"timer_subtype"}->{$typecode & $countertype_fields{"timer_subtype"}};
	my $display = $countertypes{"display"}->{$typecode & $countertype_fields{"display"}};
	my $calculation = $typecode & $countertype_fields{"calculation"};

	if( $typecode == $PERF_AVERAGE_TIMER ) {
	    $calculation = $countertypes{"calculation"}->{"delta_counter"} + $countertypes{"calculation"}->{"delta_base"};
	}

	my $val = $counter->{"Counter"};
	my $result;
	my $countersize = $counter->{"CounterSize"};
	if( ($countersize==4) && $val < 0 ) {
	    $val = $counter->{"Counter"} = 0xffffffff + $val + 1;
	}
	my( $timebase, $timebase0 );

	print "value type: $valtype, counter type: $type, display type: $display, counter rawtype: ".sprintf( "%X", $typecode )."\n" if( $BigSister::common::dl>8 );

	if( $time eq "timer_tick" ) {
	    print "time base is PerfTime\n" if( $BigSister::common::dl>9 );
	    $timebase = $current->{"PerfTime"};
	    $timebase0 = $previous->{"PerfTime"} if( $previous );
	} elsif( $time eq "timer_100ns" ) {
	    print "time base is PerfTime100nSec\n" if( $BigSister::common::dl>9 );
	    $timebase = $current->{"PerfTime100nSec"};
	    $timebase0 = $previous->{"PerfTime100nSec"} if( $previous );
	} elsif( $time eq "timer_object" ) {
	    print "time base is PerfFreq\n" if( $BigSister::common::dl>9 );
	    $perffreq = $timebase = $current->{"Objects"}->{$object}->{"PerfFreq"};
	}

	if( $calculation & $countertypes{"calculation"}->{"delta_counter"} ) {
	    next unless $previous;
	    $val -= $prevcounters->{$id}->{"Counter"};
	    if( ($val < 0) && ($countersize==4) ) {
		$val += 0xffffffff + 1;
	    }
	    
	    $timebase = $timebase-$timebase0 if( defined $timebase0 );
	    print "delta counter: delta is $val\n" if( $BigSister::common::dl>8 );
	}

	if( $valtype != "type_counter" ) {
	    $result = $val;
	}
	elsif( $type eq "counter_value" ) {
	    $result = $val;
	}
	elsif( $type eq "counter_rate" ) {
	    $result = $val / $timebase if( $timebase );
	}
	elsif( $type eq "counter_fraction" ) {
	    my $base = $counters->{$id+1}->{"Counter"};
	    if( $calculation & $countertypes{"calculation"}->{"delta_base"} ) {
		next unless( $previous );
		$base -= $prevcounters->{$id+1}->{"Counter"};
	    }
	    print "fraction val: $val, base: $base (id: ".($id+1).")\n" if( $BigSister::common::dl>9 );
	    $result = $val / $base if( $base );
	    $variables->{$varname.".fraction"} = $val;
	    $variables->{$varname.".base"} = $base;
	}
	elsif( $type eq "counter_elapsed" ) {
	    $result = $current->{"Objects"}->{$object}->{"PerfTime"}-$val;
	}
	elsif( $type eq "counter_precision" ) {
	    next unless( $previous );
	    my $base1 = $counters->{$id+1}->{"Counter"};
	    my $base0 = $prevcounters->{$id+1}->{"Counter"};
	    print "precision: $val / ($base1-$base0)\n" if( $BigSister::common::dl>9 );
	    $result = $val / ($base1-$base0) if( $base1 != $base0 );
	    $variables->{$varname.".fraction"} = $val;
	    $variables->{$varname.".base"} = $base1 -$base0;
	}

	if( defined $result ) {
	    if( $calculation & $countertypes{"calculation"}->{"inverse_counter"} ) {
		$result = 1-$result;
	    }
	    if( $display eq "display_percent" ) {
		$result *= 100;
	    }
	    elsif( $display eq "display_per_sec" ) {
		($result *= $perffreq) if( $perffreq );
		print "perffreq: $perffreq\n" if( $BigSister::common::dl > 9 );
	    }
	    elsif( $display eq "display_seconds" ) {
		($result /= $perffreq) if( $perffreq );
	    }
	    $counter->{"DisplayCounter"} = $result;
	    $variables->{$prefix.".".$name} = $result;
	}
    }
}

	    

1;

